@@ -10,8 +10,26 @@ from starlette.datastructures import UploadFile
 from pyconverters_pubmedfetcher.pubmedfetcher import InputFormat
 
 
-@pytest.mark.skip(reason="Not a test")
 def test_pubmedfetcher_ids():
+    model = PubmedFetcherConverter.get_model()
+    model_class = model.construct().__class__
+    assert model_class == PubmedFetcherParameters
+    converter = PubmedFetcherConverter()
+    parameters = PubmedFetcherParameters(input_format=InputFormat.ID_List)
+    testdir = Path(__file__).parent
+    source = Path(testdir, 'data/list.txt')
+    with source.open("rb") as fin:
+        docs: List[Document] = converter.convert(UploadFile(source.name, fin, 'text/plain'), parameters)
+        assert len(docs) == 3
+        assert docs[0].identifier == '21886606'
+        assert docs[1].identifier == '21886599'
+        assert docs[1].metadata['doi'] == '10.2174/157015911795017263'
+        assert docs[2].identifier == '21886588'
+        assert docs[2].metadata['pmc'] == '3137179'
+
+
+@pytest.mark.skip(reason="Not a test")
+def test_pubmedfetcher_allids():
     model = PubmedFetcherConverter.get_model()
     model_class = model.construct().__class__
     assert model_class == PubmedFetcherParameters
@@ -46,4 +64,4 @@ def test_pubmedfetcher_xml():
 
 
 if __name__ == '__main__':
-    test_pubmedfetcher_ids()
+    test_pubmedfetcher_allids()
